/*
 * Attiny10_Test_AVR_S5.c
 *
 * Created: 24.05.2011
 *  Author: Martin
 */ 

#include <avr/io.h>
#ifndef F_CPU
#warning "F_CPU war noch nicht definiert, wird nun mit 128kHz definiert"
#define F_CPU 128000UL     /* internen RC- Oszillator mit kalibrierten 128 kHz */
#endif
#include <util/delay.h>
#include <util/delay_basic.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <inttypes.h>


//Globale Variablen zum Gebrauch in ISR
volatile uint8_t iTimer=0;
volatile uint8_t iRichtung=0;

void Init(void){
	//internen 128 kHz Generator verwenden:
		CCP = 0xD8;
		CLKMSR = 0x01;
		CCP = 0xD8;
		CLKPSR = 0x00;

	/*Wahl des DDRB (Data Direction Register)*/

	DDRB |= (1 << DDB0); //PB0 = ein Ausgang
	PORTB |= (1 << PORTB0); //PB 0 Einschalten

	/*
	 * Einstellung der Timersettings:
	 * Es wird eine Fast-PWM-Mode initialisiert, die an TOP immer wieder aktualisiert wird.
	 *
	 * Fr FAST-PWM-Mode:
	 * TCCR0A:
	 * COM0A1	COM0A0
	 * 1		0			Clear OC0A on Compare Match set on TOP
	 *
	 * WGM02	WGM01	WGM00
	 * 0		1		1		Fast- PWM mit 0xFF fr TOP und Update an TOP( TOV Flag on MAX)
	 */
	GTCCR |= (1 << TSM);
	TCCR0A |= (1 << WGM00);
	TCCR0A &= ~(1<< WGM01);
	TCCR0B |= (1 << WGM02);
	TCCR0B &= ~(1<< WGM03);
	OCR0A = 128;
	
	TIMSK0 |= (1 << TOIE0); //Enable den Interrupt fr Timer Overflow
}

ISR(TIM0_OVF_vect){
	/*
	 * Berechnung der Hufigkeit dieses ISR:
	 * T = 256/128kHz = 2ms --> 500Hz PWM
	 */
	uint8_t iTimer_temp = iTimer;	//bergabe der globalen volatile Variablen fr
	uint8_t	iRichtung_temp = iRichtung; //sauberen bergang
	
	iTimer_temp++;	//Zum Verlangsamen der nderung wird eine Variable aufgezhlt
	if (iTimer_temp >= 10) //Also werden 10 ISR ausgefhrt, bis einmal OCR0A verndert wird.
	{
		iTimer_temp = 0; //Rcksetzen
				
		if(iRichtung_temp)//Soll nach oben gezhlt werden?
		{		
			OCR0A++;
			if (OCR0A >= 240) //Sobald OCR0A hoch genug ist, soll in die andere Richtung verndert werden
			{
				iRichtung_temp = 0; //Richtung ndern
			}		
		}
		 
		else if(!iRichtung_temp) //Soll nach unten gezhlt werden? 
		{		
			OCR0A--;
			if (OCR0A <= 10)
			{
				iRichtung_temp = 1; //Richtung ndern
			}		
		}
	}	
	iTimer = iTimer_temp;		//Rckgabe der globalen Variablen
	iRichtung = iRichtung_temp;
}//ISR
//============================================================================

void Start_Timer(void)
{
/*
* TCCR0B:
* Da ein Prescaler von 1 gebraucht wird, ist TCCR0B folgend zu beschreiben:
* CS02	CS01	CS00
* 0		0		1
* WGM 2 is initial 0!
* Da das TSM Bit gesetzt ist, und der Timer gestartet werden soll, muss dies noch gelscht werden!
*/
	//Richtig:
	TCCR0A |= (1<<COM0A1);	//Clear OC0A on Compare Match and set OC0A on TOP
	TCCR0B |= (1<<CS00);	//Start des Timers mit einem Prescaler von 1
	GTCCR &= ~(1<<TSM);		//Ende des Timer/Counter Synchronization Mode
}//Start_Timer

int main(void)
{
	cli();
	Init();	
	Start_Timer();
	sei();	
    for(;;){
		asm volatile ("nop"); //Hauptschleife soll nichts ausfhren...
		}
	return 0;
}